#ifndef __FS454_REGS_H__
#define __FS454_REGS_H__

/*
 *
 * FS454 registers
 *
 */
#define FS454_IHO		0x00	/*! Input Horizontal Offset */
#define FS454_IVO		0x02	/*! Input Vertical Offset */
#define FS454_IHW		0x04	/*! Input Horizontal Width */
#define FS454_VSC		0x06	/*! Vertical Scaling Coefficient */
#define FS454_HSC		0x08	/*! Horizontal Scaling Coefficient */
#define FS454_BYPASS		0x0A	/*! BYPASS */
#define FS454_CR		0x0C	/*! Command Register */
#define FS454_MISC		0x0E	/*! Miscellaneous Bits Register*/
#define FS454_NCON		0x10	/*! Numerator of NCO Word */
#define FS454_NCOD		0x14	/*! Denominator of NCO Word */
#define FS454_PLL_M_PUMP	0x18	/*! PLL M and Pump Control */
#define FS454_PLL_N		0x1A	/*! PLL N */
#define FS454_PLL_PDIV		0x1C	/*! PLL Post-Divider */
#define FS454_SHP		0x24	/*! Sharpness Filter */
#define FS454_FLK		0x26	/*! Filcker Filter Coefficient */
#define FS454_GPIO		0x28	/*! General Purpose I/O, Output Enab */
#define FS454_ID		0x32	/*! Part Identification Number */
#define FS454_STATUS		0x34	/*! Status Port */
#define FS454_FIFO_SP		0x36	/*! FIFO Status Port Fill/Underrun */
#define FS454_FIFO_LAT		0x38	/*! FIFO Latency */
#define FS454_CHR_FREQ		0x40	/*! Chroma Subcarrier Frequency */
#define FS454_CHR_PHASE		0x44	/*! Chroma Phase */
#define FS454_MISC_45		0x45	/*! Miscellaneous Bits Register 45 */
#define FS454_MISC_46		0x46	/*! Miscellaneous Bits Register 46 */
#define FS454_MISC_47		0x47	/*! Miscellaneous Bits Register 47 */
#define FS454_HSYNC_WID		0x48	/*! HSync Width */
#define FS454_BURST_WID		0x49	/*! Burst Width */
#define FS454_BPORCH		0x4A	/*! Back Porch Width */
#define FS454_CB_BURST		0x4B	/*! Cb Burst Amplitude */
#define FS454_CR_BURST		0x4C	/*! Cr Burst Amplitude */
#define FS454_MISC_4D		0x4D	/*! Miscellaneous Bits Register 4D */
#define FS454_BLACK_LVL		0x4E	/*! Black Level */
#define FS454_BLANK_LVL		0x50	/*! Blank Level */
#define FS454_NUM_LINES		0x57	/*! Number of Lines */
#define FS454_WHITE_LVL		0x5E	/*! White Level */
#define FS454_CB_GAIN		0x60	/*! Cb Color Saturation */
#define FS454_CR_GAIN		0x62	/*! Cr Color Saturation */
#define FS454_TINT		0x65	/*! Tint */
#define FS454_BR_WAY		0x69	/*! Width of Breezeway */
#define FS454_FR_PORCH		0x6C	/*! Front Porch */
#define FS454_NUM_PIXELS	0x71	/*! Total num. of luma/chroma Pixels */
#define FS454_1ST_LINE		0x73	/*! First Video Line */
#define FS454_MISC_74		0x74	/*! Miscellaneous Bits Register 74 */
#define FS454_SYNC_LVL		0x75	/*! Sync Level */
#define FS454_VBI_BL_LVL	0x7C	/*! VBI Blank Level */
#define FS454_SOFT_RST		0x7E	/*! Encoder Soft Reset */
#define FS454_ENC_VER		0x7F	/*! Encoder Version */
#define FS454_WSS_CONFIG	0x80	/*! WSS Configuration Register */
#define FS454_WSS_CLK		0x81	/*! WSS Clock */
#define FS454_WSS_DATAF1	0x83	/*! WSS Data Field 1 */
#define FS454_WSS_DATAF0	0x86	/*! WSS Data Field 0 */
#define FS454_WSS_LNF1		0x89	/*! WSS Line Number Field 1 */
#define FS454_WSS_LNF0		0x8A	/*! WSS Line Number Field 0 */
#define FS454_WSS_LVL		0x8B	/*! WSS Level */
#define FS454_MISC_8D		0x8D	/*! Miscellaneous Bits Register 8D */
#define FS454_VID_CNTL0		0x92	/*! Video Control 0 */
#define FS454_HD_FP_SYNC	0x94	/*! Horiz. Front Porch & HSync Width */
#define FS454_HD_YOFF_BP	0x96	/*! HDTV Lum. Offset & Back Porch */
#define FS454_SYNC_DL		0x98	/*! Sync Delay Value */
#define FS454_LD_DET		0x9C	/*! DAC Load Detect */
#define FS454_DAC_CNTL		0x9E	/*! DAC Control */
#define FS454_PWR_MGNT		0xA0	/*! Power Management */
#define FS454_RED_MTX		0xA2	/*! RGB to YCrCb Matrix Red Coeff. */
#define FS454_GRN_MTX		0xA4	/*! RGB to YCrCb Matrix Green Coeff. */
#define FS454_BLU_MTX		0xA6	/*! RGB to YCrCb Matrix Blue Coeff. */
#define FS454_RED_SCL		0xA8	/*! RGB to YCrCb Scaling Red Coeff. */
#define FS454_GRN_SCL		0xAA	/*! RGB to YCrCb Scaling Green Coeff. */
#define FS454_BLU_SCL		0xAC	/*! RGB to YCrCb Scaling Blue Coeff. */
#define FS454_CC_FIELD_1	0xAE	/*! Closed Caption Field 1 Data */
#define FS454_CC_FIELD_2	0xB0	/*! Closed Caption Field 2 Data */
#define FS454_CC_CONTROL	0xB2	/*! Closed Caption Control */
#define FS454_CC_BLANK_VALUE	0xB4	/*! Closed Caption Blanking Value */
#define FS454_CC_BLANK_SAMPLE	0xB6	/*! Closed Caption Blanking Sample */
#define FS454_HACT_ST		0xB8	/*! HDTV Horizontal Active Start */
#define FS454_HACT_WD		0xBA	/*! HDTV Horizontal Active Width */
#define FS454_VACT_ST		0xBC	/*! HDTV Veritical Active Width */
#define FS454_VACT_HT		0xBE	/*! HDTV Veritical Active Height */
#define FS454_PR_PB_SCALING	0xC0	/*! Pr and Pb Relative Scaling */
#define FS454_LUMA_BANDWIDTH	0xC2	/*! Luminance Frequency Response */
#define FS454_QPR		0xC4	/*! Quick Program Register */

/*! Command register bits */

#define CR_GCC_CK_LVL		0x2000	/*! Graphics Controller switching lev */
#define CR_P656_LVL		0x1000	/*! Pixel Port Output switching level */
#define CR_P656_IN		0x0800	/*! Pixel Port In */
#define CR_P656_OUT		0x0400	/*! Pixel Port Out */
#define CR_CBAR_480P		0x0200	/*! 480P Color Bars */
#define CR_PAL_NTSCIN		0x0100	/*! PAL or NTSC input */
#define CR_SYNC_MS		0x0080	/*! Sync Master or Slave */
#define CR_FIFO_CLR		0x0040	/*! FIFO Clear */
#define CR_CACQ_CLR		0x0020	/*! CACQ Clear */
#define CR_CDEC_BP		0x0010	/*! Chroma Decimator Bypass */
#define CR_NCO_EN		0x0002	/*! Enable NCO Latch */
#define CR_SRESET		0x0001	/*! Soft Reset */

/*! Chip ID register bits */

#define FS454_CHIP_ID		0xFE05	/*! Chip ID register expected value */

#endif
