#include <fcntl.h>
#include "linux/i2c-dev.h"
#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>
#include <linux/types.h>
#include <sys/ioctl.h>
#include "fs454_regs.h"
#include "i810_regs.h"

#define FS454_I2C_DEV	"/dev/i2c-1"
#define STV6412_I2C_DEV	"/dev/i2c-3"
#define FS454_I2C_ADDR	0x6A // SMT bus offset adress

typedef struct _video_mode{
	int htotal;
	int hactive;
	int hblank_start;
	int hblank_end;
	int hsync_start;
	int hsync_end;
	int vtotal;
	int vactive;
	int vblank_start;
	int vblank_end;
	int vsync_start;
	int vsync_end;
	long dotclock;
} video_mode;

static unsigned char fs454_defaults[] = {
		0x64, 0x00, 0x1a, 0x00, 0xd0, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x21, 0x30, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x72, 0x7c, 0x00, 0x04, 0x04, 0x1f, 0x03,
		0x00, 0x00, 0x40, 0x00, 0x0a, 0x00, 0x10, 0x00, 0x0d, 0x01, 0x00, 0x00, 0x00, 0x00, 0x08, 0x11,
		0x00, 0x0f, 0x05, 0xfe, 0x0b, 0x80, 0x00, 0x00, 0x82, 0x00, 0x0c, 0x02, 0x00, 0x00, 0x01, 0x00,
		0x2a, 0x09, 0x8a, 0xcb, 0x00, 0x00, 0x8d, 0x08, 0x7c, 0x3c, 0x9a, 0x2f, 0x21, 0x01, 0x3f, 0x00,
		0x3e, 0x03, 0x1a, 0x2a, 0x1c, 0x3d, 0x14, 0x9c, 0x01, 0x00, 0xfe, 0x7e, 0x60, 0x05, 0xc8, 0x00,
		0x9c, 0x07, 0x9c, 0x00, 0x00, 0x00, 0x08, 0x04, 0x00, 0x1a, 0x55, 0x01, 0x1e, 0x07, 0x7e, 0x02,
		0x54, 0xb1, 0x04, 0x14, 0x69, 0x10, 0x00, 0xa3, 0xc8, 0x22, 0x02, 0x22, 0x3e, 0x03, 0x00, 0x20,
		0x07, 0x2f, 0x07, 0x18, 0x00, 0x00, 0x18, 0x00, 0x00, 0x13, 0x13, 0x9c, 0x03, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x01, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe4, 0x00,
		0x00, 0x02, 0x4d, 0x00, 0x96, 0x00, 0x1d, 0x00, 0xa0, 0x00, 0xdb, 0x00, 0x7e, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00
	};



static int fs454_read(__u32 reg, __u32 * word, __u32 len)
{
	int i;
	__u8 *wp = (__u8 *) word;

	*word = 0;

	int client = open(FS454_I2C_DEV, O_RDWR);
	ioctl(client, I2C_SLAVE_FORCE, FS454_I2C_ADDR);
	if (len == 1) *word = i2c_smbus_read_byte_data(client, reg);
    else if (len == 2) *word = i2c_smbus_read_word_data(client, reg);
	else if (len == 4) { *(__u16 *) word = i2c_smbus_read_word_data(client, reg);
	*((__u16 *) word + 1) = i2c_smbus_read_word_data(client, reg + 2);
} else
 return -EINVAL;
 close(client);

}

static int fs454_write(__u32 reg, __u32 word, __u32 len)
{
	int client = open(FS454_I2C_DEV, O_RDWR);
	ioctl(client, I2C_SLAVE_FORCE, FS454_I2C_ADDR);
	if (len == 1) return i2c_smbus_write_byte_data(client, reg, (__u8) word);
	else if (len == 2) return i2c_smbus_write_word_data(client, reg, (__u16) word);
	else if (len == 4) return i2c_smbus_write_block_data(client, reg, len,(__u8 *) & word);
	else return -EINVAL;
	close(client);
}

int memfd		= -1;
unsigned char *mmio	= NULL;
off_t pciaddr		= 0xf4000000;

#define i810_readb(mmio_base, where)                     \
        *((volatile char *) (mmio_base + where))

#define i810_readw(mmio_base, where)                     \
        *((volatile short *) (mmio_base + where))

#define i810_readl(mmio_base, where)                     \
        *((volatile int *) (mmio_base + where))

#define i810_writeb(mmio_base, where, val)                              \
        *((volatile char *) (mmio_base + where)) = (volatile char) val

#define i810_writew(mmio_base, where, val)                              \
        *((volatile short *) (mmio_base + where)) = (volatile short) val

#define i810_writel(mmio_base, where, val)                              \
        *((volatile int *) (mmio_base + where)) = (volatile int) val


void memClose(void) {
	if (mmio) munmap(mmio, 512 * 1024);
	if (memfd >= 0) close(memfd);
	memfd	= -1;
	mmio	= NULL;
}

void memOpen(void) {
	memfd	= open("/dev/mem", O_RDWR);
	if (memfd < 0) {
		int i	= errno;
		perror("/dev/mem");
		if (i == EACCES && geteuid() != 0)
			fprintf(stderr, "Must have access to `/dev/mem'.\n");
		memClose();
	}

	mmio	= mmap(NULL, 512 * 1024, PROT_WRITE | PROT_READ, MAP_SHARED, memfd, pciaddr);
	if (!mmio) {
		memClose();
	}
}

__u8 i810_read_cr(int reg)
{
	i810_writeb(mmio, 0x3d4, reg);
	return (i810_readb(mmio, 0x3d5) & 0xff);
}

__u8 i810_read_sr(int reg)
{
	i810_writeb(mmio, 0x3c4, reg);
	return (i810_readb(mmio, 0x3c5) & 0xff);
}

int i815_setup(video_mode *vid_mode) {
	int char_clock;	
	int n, m, p;
	memOpen();

	if ((i810_read_sr(SR01) & 0x01) == 0)
	    char_clock = 9;
	else
	    char_clock = 8;
	    
	vid_mode->htotal = (i810_read_cr(CR00) + 5) * char_clock;
	vid_mode->hactive = (i810_read_cr(CR01) + 1) * char_clock;
	vid_mode->hblank_start = i810_read_cr(CR02) * char_clock;
	vid_mode->hblank_end = ((((i810_read_cr(CR39) & 0x01) << 6) | (i810_read_cr(CR03) & 0x1f) | ((i810_read_cr(CR05) >> 7) << 5)) + 1) * char_clock;
	vid_mode->hsync_start = i810_read_cr(CR04) * char_clock;
	vid_mode->hsync_end = (((i810_read_cr(CR39) & 0x01) << 6) | ((i810_read_cr(CR05) >> 7) << 5) | (i810_read_cr(CR05) & 0x1f)) * char_clock;
	vid_mode->vtotal;
	vid_mode->vactive;
	vid_mode->vblank_start;
	vid_mode->vsync_start;
	
	if ((i810_read_cr(CR80) & 0x01) == 0)
	{
	    vid_mode->vtotal = ((((i810_read_cr(CR07) & 0x3f) >> 5) << 9) | ((i810_read_cr(CR07) & 0x01) << 8) | i810_read_cr(CR06)) + 2;
	    vid_mode->vactive = (((i810_read_cr(CR07) & 0x7f) >> 6) << 9) | (((i810_read_cr(CR07) & 0x03) >> 1) << 8) | i810_read_cr(CR12);
	    vid_mode->vblank_start = (((i810_read_cr(CR09) & 0x3f) >> 5) << 9) | (((i810_read_cr(CR07) & 0x0f) >> 3) << 8) | i810_read_cr(CR15);
	    vid_mode->vsync_start = (((i810_read_cr(CR07) & 0xff) >> 7) << 9) | (((i810_read_cr(CR07) & 0x07) >> 2) << 8) | i810_read_cr(CR10);
	}
	
	else
	{
	    vid_mode->vtotal = (((i810_read_cr(CR30) & 0x0f) << 8) | i810_read_cr(CR06)) + 2;
	    vid_mode->vactive = (((i810_read_cr(CR31) & 0x0f) << 8) | i810_read_cr(CR12)) + 1;
	    vid_mode->vblank_start = ((i810_read_cr(CR33) & 0x0f) << 8) | i810_read_cr(CR15);
	    vid_mode->vsync_start = ((i810_read_cr(CR32) & 0x0f) << 8) | i810_read_cr(CR10);
	}

	vid_mode->vblank_end = (i810_read_cr(CR33) << 8) | i810_read_cr(CR16) + 1;
	vid_mode->vsync_end = (vid_mode->vsync_start & ~0x0f) + (i810_read_cr(CR11) & 0x0f);
	
	n = ((i810_readl(mmio, DCLK_2D) >> 16) & 0x3ff) + 2;
	m = (i810_readl(mmio, DCLK_2D) & 0x3ff) + 2;
	
	switch ((i810_readb(mmio, DCLK_0DS+2) >> 4))
	{
		case 0 :
			p = 1;
			break;
		case 1 :
			p = 2;
			break;
		case 2 :
			p = 4;
			break;
		case 3 :
			p = 8;
			break;
		case 4 :
			p = 16;
			break;
		case 5 :
			p = 32;
			break;
	}
	
	vid_mode->dotclock = (1000000000 / ((p * n * 1000000)/(4 * 24 * m))) * 1000;
	
	__u32 i815_htotal = ((vid_mode->htotal - 1) << 16) | (vid_mode->hactive - 1);
	__u32 i815_hblank = ((vid_mode->hblank_end - 1) << 16) | (vid_mode->hblank_start - 1);
	__u32 i815_hsync = ((vid_mode->hsync_end - 1) << 16) | (vid_mode->hsync_start - 1);
	__u32 i815_vtotal = ((vid_mode->vtotal - 1) << 16) | (vid_mode->vactive - 1);
	__u32 i815_vblank = ((vid_mode->vblank_end - 1) << 16) | (vid_mode->vblank_start - 1);
	__u32 i815_vsync = ((vid_mode->vsync_end - 1) << 16) | (vid_mode->vsync_start - 1);
	__u32 i815_lcdtv_c = LCDTV_C_LCDTV_ENABLE | LCDTV_C_FP_740_DATA	| LCDTV_C_ACTIVE_ORDER | LCDTV_C_BLANK_POLARITY;
	__u32 i815_ovract = ((vid_mode->hactive - 32) << 16) | (vid_mode->htotal - 32);
	__u32 i815_bclrpat = 0x00000000;
	
	i810_writel(mmio, HTOTAL, i815_htotal);
	i810_writel(mmio, HBLANK, i815_hblank);
	i810_writel(mmio, HSYNC, i815_hsync);
	i810_writel(mmio, VTOTAL, i815_vtotal);
	i810_writel(mmio, VBLANK, i815_vblank);
	i810_writel(mmio, VSYNC, i815_vsync);
	i810_writel(mmio, LCDTV_C, i815_lcdtv_c);
	i810_writel(mmio, OVRACT, i815_ovract);
	i810_writel(mmio, BCLRPAT, i815_bclrpat);
	
	memClose();
	
	return 0;
}

int fs454_setup(video_mode *vid_mode)
{
	int i;
	int tv_htotal = 864;
	int tv_vtotal = 625;
	int tv_hactive = 720;
	int tv_vactive = 576;
	int vsc, hsc;

	fs454_defaults[0x04] = (vid_mode->hactive+64) & 0xff;
	fs454_defaults[0x05] = ((vid_mode->hactive+64) >> 8) & 0xff;
		
	if (tv_vtotal > vid_mode->vtotal) {
		/* Upscaling */
		vsc = ((((float)tv_vtotal/(float)vid_mode->vtotal)-1)*65536);
		/* For upscaling, adjust FIFO_LAT (FIFO latency) */
		fs454_defaults[0x38] = 0x82;
	}
	
	else {
		/* Downscaling */
		vsc = ((((float)tv_vtotal/(float)vid_mode->vtotal))*65536);
	}
	fs454_defaults[0x06] = (vsc)&0xFF;
	fs454_defaults[0x07] = (vsc>>8)&0xFF;

	hsc = 128*((float)tv_hactive/(float)vid_mode->htotal-1);
	if (tv_hactive > vid_mode->htotal) {
		/* Upscaling */
		fs454_defaults[0x08] = 0;
		fs454_defaults[0x09] = hsc&0xFF;
	}
	
	else {  /* Downscaling */
		hsc = 256 + hsc;
		fs454_defaults[0x08] = hsc&0xFF;
		fs454_defaults[0x09] = 0;
	}
	
	if (!focus_calc_pll_settings(vid_mode))
	{
		printf ("Keinen gltigen Mode gefunden\n");
		return 1;
	}
		
	
	
	// Focus Reset
	while (fs454_write(FS454_SOFT_RST, 0x01, 1));	
	while (fs454_write(FS454_SOFT_RST, 0x00, 1));	

	// Soft Reset Bit setzen
	while (fs454_write(FS454_CR, 0x2111, 2));

	for (i=0;i<0xc4;i++)
	{
		while (fs454_write(i,fs454_defaults[i], 1));
	}

	//PLL nderungen bernehmen
	while (fs454_write(FS454_CR, 0x2112, 2));
	while (fs454_write(FS454_CR, 0x2110, 2));

	//Soft Reset Bit lschen
	while (fs454_write(FS454_CR, 0x2110, 2));

	return 0;
}

int focus_calc_pll_settings(video_mode *vid_mode)
{
	int tv_htotal = 864;
	int tv_vtotal = 625;
	int tv_hactive = 720;
	int tv_vactive = 576;
	int m, n;
	int pll_multiplier;
	long ncon, ncod;
	ncon = vid_mode->htotal * vid_mode->vtotal;
	 
	 //Multipliers between 1 and 6 are the limit as output clock cant be >150MHz
	 //The lower the multiplier, the more stable the PLL (theoretically)
	 for (pll_multiplier=4; pll_multiplier<6; pll_multiplier++) {
	 	long nco_out_clk;
	 	ncod = tv_htotal * tv_vtotal * pll_multiplier;
	 	//NCO output clock is the reference clock (27MHz) multiplied by
	 	//the ncon/ncod fraction.
	 	nco_out_clk = 27000000*(ncon/(float)ncod);
	 	for (n=2; n<270;++n) {
	 		//PLL input clock is NCO output clock divided by N
	 		//Valid range is 100kHz to 1000kHz
	 		long pll_in_clk = nco_out_clk/n;
	 		if ( pll_in_clk >=100000 && pll_in_clk <=1000000) {
  	 			for (m=2; m<3000;++m) {
	 				//PLL output clock is PLL input clock multiplied
	 				//by M. Valid range is 100MHz to 300MHz
	 				long pll_out_clk = pll_in_clk * m;
	 				if (pll_out_clk >=100000000 && pll_out_clk <= 300000000) {
	 					//Output clocks are PLL output clock divided by P.
	 					//Valid range is anything LESS than 150MHz, but
	 					//it must match the incoming pixel clock rate.
	 					long output_clk = pll_out_clk/pll_multiplier;
	 					if (output_clk == vid_mode->dotclock || output_clk+1 == vid_mode->dotclock || output_clk-1 == vid_mode->dotclock) {
	 						//Got it - the pll is now correctly aligned
	 						//Set up the PLL registers
	 						fs454_defaults[0x10] = (ncon)&0xFF;
	 						fs454_defaults[0x11] = (ncon>>8)&0xFF ;
	 						fs454_defaults[0x12] = (ncon>>16)&0xFF ;
	 						fs454_defaults[0x13] = (ncon>>24)&0xFF ;
	 						fs454_defaults[0x14] = (ncod)&0xFF ;
	 						fs454_defaults[0x15] = (ncod>>8)&0xFF ;
	 						fs454_defaults[0x16] = (ncod>>16)&0xFF ;
	 						fs454_defaults[0x17] = (ncod>>24)&0xFF ;
	 
	 						fs454_defaults[0x18] = (m-17)&0xFF;
	 						fs454_defaults[0x19] = (((m-17)>>8) | 0x00 )&0xFF;
	 						fs454_defaults[0x1A] = (n-1)&0xFF ;
	 						fs454_defaults[0x1B] = ((n-1)>>8)&0xFF ;
	 						fs454_defaults[0x1C] = (pll_multiplier-1)&0xFF;
	 						fs454_defaults[0x1D] = (pll_multiplier-1)&0xFF;
	 						return 1;
	 					}
					}
				}
			}
		}
	}
	//Seems no valid solution was possible
	return 0;
}

void stv6412a_setup(void) {
	int client = open(STV6412_I2C_DEV, O_RDWR);
	ioctl(client, I2C_SLAVE_FORCE, 0x4a);
	i2c_smbus_write_byte_data(client, 0x03, 0xb6);
	close(client);
}

int main(int argc, char *argv[]) {
	video_mode vid_mode;
	i815_setup(&vid_mode);
	fs454_setup(&vid_mode);
	stv6412a_setup();
}

